package Net::Async::Trello::Generated::Member;

use strict;
use warnings;

our $VERSION = '0.005'; # VERSION

use parent qw(Net::Async::Trello::Common);

=head1 NAME

Net::Async::Trello::Generated::Member

=head1 DESCRIPTION

Autogenerated module.

=cut

use URI ();

=head1 METHODS

=cut

=head2 url

Provides an accessor for C<url>.

=cut

sub url { shift->{url} }

=head2 member_type

Provides an accessor for C<memberType>.

=cut

sub member_type { shift->{memberType} }

=head2 id_organizations

Provides an accessor for C<idOrganizations>.

=cut

sub id_organizations { shift->{idOrganizations} }

=head2 id_boards

Provides an accessor for C<idBoards>.

=cut

sub id_boards { shift->{idBoards} }

=head2 initials

Provides an accessor for C<initials>.

=cut

sub initials { shift->{initials} }

=head2 premium_features

Provides an accessor for C<premiumFeatures>.

=cut

sub premium_features { shift->{premiumFeatures} }

=head2 id

Provides an accessor for C<id>.

=cut

sub id { shift->{id} }

=head2 login_types

Provides an accessor for C<loginTypes>.

=cut

sub login_types { shift->{loginTypes} }

=head2 id_prem_orgs_admin

Provides an accessor for C<idPremOrgsAdmin>.

=cut

sub id_prem_orgs_admin { shift->{idPremOrgsAdmin} }

=head2 id_enterprises_admin

Provides an accessor for C<idEnterprisesAdmin>.

=cut

sub id_enterprises_admin { shift->{idEnterprisesAdmin} }

=head2 email

Provides an accessor for C<email>.

=cut

sub email { shift->{email} }

=head2 id_enterprise

Provides an accessor for C<idEnterprise>.

=cut

sub id_enterprise { shift->{idEnterprise} }

=head2 status

Provides an accessor for C<status>.

=cut

sub status { shift->{status} }

=head2 prefs

Provides an accessor for C<prefs>.

=cut

sub prefs { shift->{prefs} }

=head2 id_boards_pinned

Provides an accessor for C<idBoardsPinned>.

=cut

sub id_boards_pinned { shift->{idBoardsPinned} }

=head2 bio

Provides an accessor for C<bio>.

=cut

sub bio { shift->{bio} }

=head2 confirmed

Provides an accessor for C<confirmed>.

=cut

sub confirmed { shift->{confirmed} }

=head2 full_name

Provides an accessor for C<fullName>.

=cut

sub full_name { shift->{fullName} }

=head2 gravatar_hash

Provides an accessor for C<gravatarHash>.

=cut

sub gravatar_hash { shift->{gravatarHash} }

=head2 trophies

Provides an accessor for C<trophies>.

=cut

sub trophies { shift->{trophies} }

=head2 username

Provides an accessor for C<username>.

=cut

sub username { shift->{username} }

=head2 products

Provides an accessor for C<products>.

=cut

sub products { shift->{products} }

=head2 one_time_messages_dismissed

Provides an accessor for C<oneTimeMessagesDismissed>.

=cut

sub one_time_messages_dismissed { shift->{oneTimeMessagesDismissed} }

=head2 uploaded_avatar_hash

Provides an accessor for C<uploadedAvatarHash>.

=cut

sub uploaded_avatar_hash { shift->{uploadedAvatarHash} }

=head2 avatar_source

Provides an accessor for C<avatarSource>.

=cut

sub avatar_source { shift->{avatarSource} }

=head2 bio_data

Provides an accessor for C<bioData>.

=cut

sub bio_data { shift->{bioData} }

=head2 avatar_hash

Provides an accessor for C<avatarHash>.

=cut

sub avatar_hash { shift->{avatarHash} }

1;

