package Net::Async::Trello::Member;

use strict;
use warnings;

our $VERSION = '0.005'; # VERSION

use parent qw(Net::Async::Trello::Generated::Member);

=head1 NAME

Net::Async::Trello::Member

=head1 DESCRIPTION

Autogenerated module.

=cut

1;

