package Net::Async::Trello::Generated::CardAction;

use strict;
use warnings;

our $VERSION = '0.005'; # VERSION

use parent qw(Net::Async::Trello::Common);

=head1 NAME

Net::Async::Trello::Generated::CardAction

=head1 DESCRIPTION

Autogenerated module.

=cut

use URI ();

=head1 METHODS

=cut

=head2 badges

Provides an accessor for C<badges>.

=cut

sub badges { shift->{badges} }

=head2 check_item_states

Provides an accessor for C<checkItemStates>.

=cut

sub check_item_states { shift->{checkItemStates} }

=head2 closed

Provides an accessor for C<closed>.

=cut

sub closed { shift->{closed} }

=head2 date_last_activity

Provides an accessor for C<dateLastActivity>.

=cut

sub date_last_activity { shift->{dateLastActivity} }

=head2 desc

Provides an accessor for C<desc>.

=cut

sub desc { shift->{desc} }

=head2 desc_data

Provides an accessor for C<descData>.

=cut

sub desc_data { shift->{descData} }

=head2 due

Provides an accessor for C<due>.

=cut

sub due { shift->{due} }

=head2 email

Provides an accessor for C<email>.

=cut

sub email { shift->{email} }

=head2 id_attachment_cover

Provides an accessor for C<idAttachmentCover>.

=cut

sub id_attachment_cover { shift->{idAttachmentCover} }

=head2 id_board

Provides an accessor for C<idBoard>.

=cut

sub id_board { shift->{idBoard} }

=head2 id_checklists

Provides an accessor for C<idChecklists>.

=cut

sub id_checklists { shift->{idChecklists} }

=head2 id_labels

Provides an accessor for C<idLabels>.

=cut

sub id_labels { shift->{idLabels} }

=head2 id_list

Provides an accessor for C<idList>.

=cut

sub id_list { shift->{idList} }

=head2 id_members

Provides an accessor for C<idMembers>.

=cut

sub id_members { shift->{idMembers} }

=head2 id_members_voted

Provides an accessor for C<idMembersVoted>.

=cut

sub id_members_voted { shift->{idMembersVoted} }

=head2 id_short

Provides an accessor for C<idShort>.

=cut

sub id_short { shift->{idShort} }

=head2 labels

Provides an accessor for C<labels>.

=cut

sub labels { shift->{labels} }

=head2 manual_cover_attachment

Provides an accessor for C<manualCoverAttachment>.

=cut

sub manual_cover_attachment { shift->{manualCoverAttachment} }

=head2 name

Provides an accessor for C<name>.

=cut

sub name { shift->{name} }

=head2 pos

Provides an accessor for C<pos>.

=cut

sub pos { shift->{pos} }

=head2 short_link

Provides an accessor for C<shortLink>.

=cut

sub short_link { shift->{shortLink} }

=head2 short_url

Provides an accessor for C<shortUrl>.

=cut

sub short_url { shift->{shortUrl} }

=head2 subscribed

Provides an accessor for C<subscribed>.

=cut

sub subscribed { shift->{subscribed} }

=head2 url

Provides an accessor for C<url>.

=cut

sub url { shift->{url} }

1;

