package Net::Async::Trello::List;

use strict;
use warnings;

our $VERSION = '0.005'; # VERSION

use parent qw(Net::Async::Trello::Generated::List);

=head1 NAME

Net::Async::Trello::List

=head1 DESCRIPTION

Autogenerated module.

=cut

sub cards {
	my ($self, %args) = @_;
    $self->trello->api_get_list(
		uri => 'lists/' . $self->id . '/cards',
        class => 'Net::Async::Trello::Card',
        extra => {
            list  => $self,
        },
    )
}

1;

