package Net::Async::Trello::Generated::List;

use strict;
use warnings;

our $VERSION = '0.005'; # VERSION

use parent qw(Net::Async::Trello::Common);

=head1 NAME

Net::Async::Trello::Generated::List

=head1 DESCRIPTION

Autogenerated module.

=cut

=head1 METHODS

=cut

=head2 id

Provides an accessor for C<id>.

=cut

sub id { shift->{id} }

=head2 name

Provides an accessor for C<name>.

=cut

sub name { shift->{name} }

=head2 cards

Provides an accessor for C<cards>.

=cut

sub cards { shift->{cards} }

1;

